# The LearningOnline Network
# "About Me" Personal Information
#
# $Id: lonaboutme.pm,v 1.27 2004/05/12 19:25:15 albertel Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#

package Apache::lonaboutme;

use strict;
use Apache::Constants qw(:common);
use Apache::loncommon;
use Apache::lonnet;
use Apache::lontexconvert;
use Apache::lonfeedback;
use Apache::lonlocal;

sub handler {
    my $r = shift;
    &Apache::loncommon::content_type($r,'text/html');
    $r->send_http_header;
    return OK if $r->header_only;
    my $target=$ENV{'form.grade_target'};
# ------------------------------------------------------------ Print the screen
    if ($target ne 'tex') {
	$r->print(<<ENDDOCUMENT);
<html>
<head>
<title>The LearningOnline Network with CAPA</title>
ENDDOCUMENT
    } else {
	$r->print(&Apache::lonprintout::print_latex_header($ENV{'form.latex_type'}));
    }
    my (undef,undef,$cdom,$cnum)=split(/\//,$r->uri);
# Is this even a user?
    if (&Apache::lonnet::homeserver($cnum,$cdom) eq 'no_host') {
	$r->print('</head><body>'.
		  &mt('No user information available').'</body></html>');
        return OK;
    }
# --------------------------------------------------------- The syllabus fields
    my %syllabusfields=&Apache::lonlocal::texthash(
       'aaa_contactinfo'   => 'Contact Information',
       'bbb_aboutme'       => 'About Me',
       'ccc_webreferences' => 'Web References');

# ------------------------------------------------------------ Get Query String
    &Apache::loncommon::get_unprocessed_cgi
                            ($ENV{'QUERY_STRING'},['forcestudent','register']);
# ----------------------------------------------------- Force menu registration
    my $addentries='';
    if ($ENV{'form.register'}) {
       $addentries=' onLoad="'.&Apache::lonmenu::loadevents().
	   '" onUnload="'.&Apache::lonmenu::unloadevents().'"';
       $r->print(&Apache::lonmenu::registerurl(1));
    }
# --------------------------------------------------------------- Force Student
    my $forcestudent='';
    if ($ENV{'form.forcestudent'}) { $forcestudent='student'; };
       
# --------------------------------------- There is such a user, get environment
    my %courseenv=&Apache::lonnet::dump('environment',$cdom,$cnum);
    if ($target ne 'tex') {
	$r->print('</head>'.&Apache::loncommon::bodytag
                  ("Personal Information",$forcestudent,$addentries,'',$cdom,
                   $ENV{'form.register'}));
	$r->print('<h1>'.&Apache::loncommon::plainname($cnum,$cdom).'</h1>');
    } else {
	$r->print('\noindent{\large\textbf{'.&Apache::loncommon::plainname($cnum,$cdom).'}}\\\\\\\\');
    }
    if ($courseenv{'nickname'}) {
       $r->print(
         '<h2>&quot;'.$courseenv{'nickname'}.
         '&quot;</h2>');
    }
    if ($target ne 'tex') {
	$r->print('<h3>'.$Apache::lonnet::domaindescription{$cdom}.'</h3>'.
		  '<p>'.&Apache::loncommon::messagewrapper('Send me a message',$cnum,$cdom).'</p>');
    } else {
	$r->print('\textbf{'.$Apache::lonnet::domaindescription{$cdom}.'}\\\\');
    }
    my %syllabus=&Apache::lonnet::dump('aboutme',$cdom,$cnum);
    my $allowed=0;

# does this user have privileges to post, etc?

       $allowed=(($ENV{'user.name'} eq $cnum) && 
		 ($ENV{'user.domain'} eq $cdom));
       if ($forcestudent or $target eq 'tex') { $allowed=0; }
 
       if ($allowed) {
	   if ($target ne 'tex') {
	       $r->print('<p><b>'.&mt('Privacy Note').':</b> '.
                         &mt('The information you submit can be viewed by anybody who is logged into LON-CAPA. Do not provide information that you are not ready to share publicly.').
                         '</p>'.
                         &Apache::loncommon::help_open_topic('Uploaded_Templates_TextBoxes','Help with filling in text boxes').'</p><p><a href="'.$r->uri.'?forcestudent=1">Show Public View</a>'.
                         &Apache::loncommon::help_open_topic('Uploaded_Templates_PublicView').'</p>');
	   } else {
	       $r->print(' \textbf{'.&mt('Privacy Note').'}: '.
                         &mt('The information you submit can be viewed by anybody who is logged into LON-CAPA. Do not provide information that you are not ready to share publicly.').'\\\\\\\\');
	   }
      }
      if (($ENV{'form.uploaddoc.filename'}) &&
          ($ENV{'form.storeupl'}) && ($allowed)) {
 	  if ($ENV{'form.uploaddoc.filename'}=~/\.(gif|jpg|png|jpeg)$/i) {
	      if ($syllabus{'uploaded.photourl'}) {
		  &Apache::lonnet::removeuploadedurl($syllabus{'uploaded.photourl'});
	      }
	      $syllabus{'uploaded.photourl'}=
                 &Apache::lonnet::userfileupload('uploaddoc',undef,'aboutme');
 	  }
          $syllabus{'uploaded.lastmodified'}=time;
          &Apache::lonnet::put('aboutme',\%syllabus,$cdom,$cnum);
       }
       if (($allowed) && ($ENV{'form.storesyl'})) {
	   foreach (keys %syllabusfields) {
               my $field=$ENV{'form.'.$_};
               $field=~s/\s+$//s;
               $field=&Apache::lonfeedback::clear_out_html($field,
                                                           $ENV{'user.adv'});
	       $syllabus{$_}=$field;
           }
           $syllabus{'uploaded.lastmodified'}=time;
           &Apache::lonnet::put('aboutme',\%syllabus,$cdom,$cnum);
       }

# ---------------------------------------------------------------- Get syllabus
    if (($syllabus{'uploaded.lastmodified'}) || ($allowed)) {
       my $lastmod=$syllabus{'uploaded.lastmodified'};
       $lastmod=($lastmod?&Apache::lonlocal::locallocaltime($lastmod):&mt('never'));
       $r->print(&mt('Last updated').': '.$lastmod);
       if ($syllabus{'uploaded.photourl'}) {
	   &Apache::lonnet::allowuploaded('/adm/aboutme',
					  $syllabus{'uploaded.photourl'});
	   my $image='<img src="'.$syllabus{'uploaded.photourl'}.'"
                           align="right" />';
	   if ($target eq 'tex') {
	       $image=&Apache::lonxml::xmlparse($r,'tex',$image);
	   }
	   $r->print($image);
       }
       if ($allowed) {
           $r->print(
	 '<form method="post" enctype="multipart/form-data">'.
         '<h3>'.&mt('Upload a Photo').'</h3>'.
         '<input type="file" name="uploaddoc" size="50">'.
         '<input type="submit" name="storeupl" value="Upload">'.
	 '</form><form method="post">');
       }
       foreach (sort keys %syllabusfields) {
          if (($syllabus{$_}) || ($allowed)) {
              my $message=$syllabus{$_};
              $message=~s/\n/\<br \/\>/g;
              $message
             =~s/(http\:\/\/[^\s]+)/\<a href=\"$1\"\>\<tt\>$1\<\/tt\>\<\/a\>/g;
	      $message=&Apache::lontexconvert::msgtexconverted($message);
	      if ($target ne 'tex') {
		  $r->print('<h3>'.$syllabusfields{$_}.'</h3><blockquote>'.
                            $message.'</blockquote>');
	      } else {
		     $r->print('\\\\\textbf{'.$syllabusfields{$_}.'}\\\\'.
			       &Apache::lonxml::xmlparse($r,'tex',$message).'\\\\');
	      }
              if ($allowed) {
                 $r->print('<br /><textarea cols="80" rows="6" name="'.$_.'">'.
			   $syllabus{$_}.
           '</textarea><input type="submit" name="storesyl" value="'.
			   &mt('Store').'" />');
	      }
	  }
       }
       if ($allowed) {
	   $r->print('</form>');
       }
       if ($target ne 'tex') {$r->print('</p>');} else {$r->print('\\\\');}
    } else {
       $r->print('<p>'.&mt('No personal information provided').'.</p>');
    }
    if ($ENV{'request.course.id'}) {
	if (&Apache::lonnet::allowed('srm',$ENV{'request.course.id'})) {
	    if ($target ne 'tex') {
		$r->print('<hr /><h3>'.
                          &mt('User Notes, Records of Face-To-Face Discussions, and Critical Messages in Course').'</h3>'.
                          &mt('Shared by course faculty and staff').
                          &Apache::loncommon::help_open_topic("Course_Face_To_Face_Records,Course_Critical_Message").
'<br />');
	    &Apache::lonmsg::disfacetoface($r,$cnum,$cdom);
            $r->print(&Apache::loncommon::noteswrapper('Add Records',$cnum,$cdom));
	    } else {
		$r->print('\\\\\textbf{'.&mt('User Notes, Records of Face-To-Face Discussions, and Critical Messages in Course').'}\\\\'.&mt('Shared by course faculty and staff').'\\\\\\\\');
		&Apache::lonmsg::disfacetoface($r,$cnum,$cdom);
	    }
        }
    }
    if ($target ne 'tex') {$r->print('</body></html>');} else {$r->print('\end{document}');}
    return OK;
} 

1;
__END__
